==> easy-batch-core-7.0.0-sources.jar: org/jeasy/batch/core/jmx/JobMonitor.java <==
/*
 * The MIT License
 *
 *   Copyright (c) 2020, Mahmoud Ben Hassine (mahmoud.benhassine@icloud.com)
 *
 *   Permission is hereby granted, free of charge, to any person obtaining a copy
 *   of this software and associated documentation files (the "Software"), to deal
 *   in the Software without restriction, including without limitation the rights
 *   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *   copies of the Software, and to permit persons to whom the Software is
 *   furnished to do so, subject to the following conditions:
 *
 *   The above copyright notice and this permission notice shall be included in
 *   all copies or substantial portions of the Software.
 *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 *   THE SOFTWARE.
 */
package org.jeasy.batch.core.jmx;

import org.jeasy.batch.core.job.Job;
import org.jeasy.batch.core.job.JobReport;
import org.jeasy.batch.core.util.Utils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.management.*;
import java.lang.management.ManagementFactory;
import java.sql.Timestamp;
import java.time.LocalDateTime;

/**
 * JMX MBean implementation of {@link JobMonitorMBean}.
 *
 * @author Mahmoud Ben Hassine (mahmoud.benhassine@icloud.com)
 */
public class JobMonitor extends NotificationBroadcasterSupport implements JobMonitorMBean {

    private static final Logger LOGGER = LoggerFactory.getLogger(JobMonitor.class.getName());

    /**
     * JMX notification sequence number.
     */
    private long sequenceNumber = 1;

    /**
     * The batch report holding data exposed as JMX attributes.
     */
    private JobReport jobReport;

    public JobMonitor(final JobReport jobReport) {
        this.jobReport = jobReport;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getJobName() {
        return jobReport.getJobName();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public long getReadCount() {
        return jobReport.getMetrics().getReadCount();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public long getFilterCount() {
        return jobReport.getMetrics().getFilterCount();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public long getErrorCount() {
        return jobReport.getMetrics().getErrorCount();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public long getWriteCount() {
        return jobReport.getMetrics().getWriteCount();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getStartTime() {
        return Utils.formatTime(jobReport.getMetrics().getStartTime());
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getEndTime() {
        return (jobReport.getMetrics().getEndTime() == null) ? "" : Utils.formatTime(jobReport.getMetrics().getEndTime());
    }

    @Override
    public String getJobStatus() {
        return jobReport.getStatus().name();
    }

    public void notifyJobReportUpdate() {
        Notification notification = new AttributeChangeNotification(
                this,
                sequenceNumber++,
                Timestamp.valueOf(LocalDateTime.now()).getTime(),
                "job report updated",
                "JobReport",
                JobReport.class.getName(),
                null, //no need for old value
                jobReport);
        sendNotification(notification);
    }

    public void registerJmxMBeanFor(Job job) {
        LOGGER.debug("Registering JMX MBean for job '{}'", job.getName());
        MBeanServer mbs = ManagementFactory.getPlatformMBeanServer();
        ObjectName name;
        try {
            name = new ObjectName(JobMonitorMBean.JMX_MBEAN_NAME + "name=" + job.getName());
            if (!mbs.isRegistered(name)) {
                mbs.registerMBean(this, name);
                LOGGER.debug("JMX MBean registered successfully as: {}", name.getCanonicalName());
            }
        } catch (Exception e) {
            LOGGER.error("Unable to register MBean for job '{}'", job.getName(), e);
        }
    }
}

==> easy-batch-core-7.0.2-sources.jar: org/jeasy/batch/core/jmx/JobMonitor.java <==
/*
 * The MIT License
 *
 *   Copyright (c) 2021, Mahmoud Ben Hassine (mahmoud.benhassine@icloud.com)
 *
 *   Permission is hereby granted, free of charge, to any person obtaining a copy
 *   of this software and associated documentation files (the "Software"), to deal
 *   in the Software without restriction, including without limitation the rights
 *   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *   copies of the Software, and to permit persons to whom the Software is
 *   furnished to do so, subject to the following conditions:
 *
 *   The above copyright notice and this permission notice shall be included in
 *   all copies or substantial portions of the Software.
 *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 *   THE SOFTWARE.
 */
package org.jeasy.batch.core.jmx;

import org.jeasy.batch.core.job.Job;
import org.jeasy.batch.core.job.JobReport;
import org.jeasy.batch.core.util.Utils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.management.AttributeChangeNotification;
import javax.management.MBeanServer;
import javax.management.Notification;
import javax.management.NotificationBroadcasterSupport;
import javax.management.ObjectName;

import java.lang.management.ManagementFactory;
import java.sql.Timestamp;
import java.time.LocalDateTime;

/**
 * JMX MBean implementation of {@link JobMonitorMBean}.
 *
 * @author Mahmoud Ben Hassine (mahmoud.benhassine@icloud.com)
 */
public class JobMonitor extends NotificationBroadcasterSupport implements JobMonitorMBean {

    private static final Logger LOGGER = LoggerFactory.getLogger(JobMonitor.class.getName());

    /**
     * JMX notification sequence number.
     */
    private long sequenceNumber = 1;

    /**
     * The batch report holding data exposed as JMX attributes.
     */
    private JobReport jobReport;

    public JobMonitor(final JobReport jobReport) {
        this.jobReport = jobReport;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getJobName() {
        return jobReport.getJobName();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public long getReadCount() {
        return jobReport.getMetrics().getReadCount();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public long getFilterCount() {
        return jobReport.getMetrics().getFilterCount();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public long getErrorCount() {
        return jobReport.getMetrics().getErrorCount();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public long getWriteCount() {
        return jobReport.getMetrics().getWriteCount();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getStartTime() {
        return Utils.formatTime(jobReport.getMetrics().getStartTime());
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getEndTime() {
        return (jobReport.getMetrics().getEndTime() == null) ? "" : Utils.formatTime(jobReport.getMetrics().getEndTime());
    }

    @Override
    public String getJobStatus() {
        return jobReport.getStatus().name();
    }

    public void notifyJobReportUpdate() {
        Notification notification = new AttributeChangeNotification(
                this,
                sequenceNumber++,
                Timestamp.valueOf(LocalDateTime.now()).getTime(),
                "job report updated",
                "JobReport",
                JobReport.class.getName(),
                null, //no need for old value
                jobReport);
        sendNotification(notification);
    }

    public void registerJmxMBeanFor(Job job) {
        LOGGER.debug("Registering JMX MBean for job '{}'", job.getName());
        MBeanServer mbs = ManagementFactory.getPlatformMBeanServer();
        ObjectName name;
        try {
            name = new ObjectName(JobMonitorMBean.JMX_MBEAN_NAME + "name=" + job.getName());
            if (!mbs.isRegistered(name)) {
                mbs.registerMBean(this, name);
                LOGGER.debug("JMX MBean registered successfully as: {}", name.getCanonicalName());
            }
        } catch (Exception e) {
            LOGGER.error("Unable to register MBean for job '{}'", job.getName(), e);
        }
    }
}
